from __future__ import print_function
import argparse
import os
import numpy
import random
import torch
import torch.nn as nn
import torch.backends.cudnn as cudnn
import torchvision.transforms as transforms
import torchvision.utils as vutils
from torch.autograd import Variable
from utils.dataset import DATASET 
from model.fixup_net import FIXUPNET
from utils.fitsFun import DATASET_fits
from utils.fitsFun import LoadSaveFits

parser = argparse.ArgumentParser(description='train pix2pix model')
parser.add_argument('--batchSize', type=int, default=1, help='with batchSize=1 equivalent to instance normalization.')
parser.add_argument('--ngf', type=int, default=64*4)
parser.add_argument('--ndf', type=int, default=64*4)
parser.add_argument('--cuda', action='store_true', help='enables cuda')
parser.add_argument('--outf', default='samples/', help='folder to output images and model checkpoints')
parser.add_argument('--manualSeed', type=int, help='manual seed')
parser.add_argument('--dataPath', default='./DATA/', help='path to training images')
parser.add_argument('--loadSize', type=int, default=20, help='scale image to this size')
parser.add_argument('--fineSize', type=int, default=32, help='random crop image to this size')
parser.add_argument('--flip', type=int, default=0, help='1 for flipping image randomly, 0 for not')
parser.add_argument('--input_nc', type=int, default=3721, help='channel number of input image')
parser.add_argument('--output_nc', type=int, default=3721, help='channel number of output image')
parser.add_argument('--G_AB', default='./checkpoints/G_AB_20000.pth', help='path to pre-trained G_AB')
parser.add_argument("--gpus", default="0", type=str, help="gpu ids (default: 0)")
opt = parser.parse_args()
print(opt)

try:
    os.makedirs(opt.outf)
except OSError:
    pass

if opt.manualSeed is None:
    opt.manualSeed = random.randint(1, 10000)
print("Random Seed: ", opt.manualSeed)
random.seed(opt.manualSeed)
torch.manual_seed(opt.manualSeed)
if opt.cuda:
    print("=> use gpu id: '{}'".format(opt.gpus))
    os.environ["CUDA_VISIBLE_DEVICES"] = opt.gpus
    if not torch.cuda.is_available():
            raise Exception("No GPU found or Wrong gpu id, please run without --cuda")

opt.seed = random.randint(1, 10000)
print("Random Seed: ", opt.seed)
torch.manual_seed(opt.seed)

cudnn.benchmark = True
##########   DATASET   ###########
listp = os.listdir(opt.dataPath)
listp.sort()

datasetA = DATASET_fits(opt.dataPath,opt.fineSize)
loader_A= torch.utils.data.DataLoader(dataset=datasetA,
                                       batch_size=opt.batchSize,
                                       shuffle=False,
                                       num_workers=2)
loaderA = iter(loader_A)

###########  MODEL   ###########
ndf = opt.ndf
ngf = opt.ngf
G_AB = FIXUPNET(opt.input_nc, opt.output_nc, opt.ngf)

from collections import OrderedDict
if(opt.G_AB != ''):
    print('Warning! Loading pre-trained weights.')
    G_AB.load_state_dict(torch.load(opt.G_AB))
    
else:
    print('ERROR! G_AB and G_BA must be provided!')

if(opt.cuda):
    G_AB.cuda()

###########   GLOBAL VARIABLES   ###########
input_nc = opt.input_nc
output_nc = opt.output_nc
fineSize = opt.fineSize

real_A = torch.FloatTensor(opt.batchSize, input_nc, fineSize, fineSize)
real_B = torch.FloatTensor(opt.batchSize, output_nc, fineSize, fineSize)

real_A = Variable(real_A)
real_B = Variable(real_B)

if(opt.cuda):
    real_A = real_A.cuda()
    real_B = real_B.cuda()

###########   Testing    ###########
def test():

    for i in range(0,50,1):
        imgA = loaderA.next() 
        real_A.data.resize_(imgA[:,0:input_nc,:,:].size()).copy_(imgA[:,0:input_nc,:,:])
        real_B.data.resize_(imgA[:,input_nc:input_nc*2,:,:].size()).copy_(imgA[:,input_nc:input_nc*2,:,:])
        print(i)   
        AB = G_AB(real_A) 
        LoadSaveFits.save_fit(real_B.data,listp[i][:-5]+'_b','./out_picture/out_image_test/')
        LoadSaveFits.save_fit(AB.data,listp[i][:-5]+'_ab','./out_picture/out_image_test/')
test()
