import PIL.Image as Image
from astropy.io import fits
import torchvision.transforms as transforms
import numpy as np
import torch
import os
import h5py
import torch.utils.data as Data
from skimage import exposure
import random
class LoadSaveFits:
    def __init__(self,path,img,name):
        self.path = path
        self.img = img
        self.name = name
        
    def norm(img):
        img = (img - np.min(img))/(np.max(img) - np.min(img)) #normalization
        img -= np.mean(img)  # take the mean
        img /= np.std(img)  #standardization
        img = np.array(img,dtype='float32')
        return img
 
    def norm2(img,z):
        for i in range(z):
            if np.max(img[i])>0:
                img[i] = (img[i] - np.min(img[i]))/(np.max(img[i]) - np.min(img[i])) #normalization
                img[i] -= np.mean(img[i])  # take the mean
                img[i] /= np.std(img[i])  #standardization
                img[i] = np.array(img[i],dtype='float32')
        return img
       
    def read_fits(path):
        hdu = fits.open(path)
        img = hdu[0].data
        img = np.array(img,dtype = np.float32)
        hdu.close()
        return img
    
    def save_fit_cpu(img,name,path):
        if os.path.exists(path + name+'.fits'):
            os.remove(path + name+'.fits')
        grey=fits.PrimaryHDU(img)
        greyHDU=fits.HDUList([grey])
        greyHDU.writeto(path + name+'.fits')
        
    def save_fit(img,name,path):
        if torch.cuda.is_available(): 
            img = torch.Tensor.cpu(img)
            img = img.data.numpy()
            IMG = img[0,:,:,:]
        else:
            img = np.array(img)
        if os.path.exists(path + name+'.fits'):
            os.remove(path + name+'.fits')
        grey=fits.PrimaryHDU(IMG)
        greyHDU=fits.HDUList([grey])
        greyHDU.writeto(path + name+'.fits')

# load data of cycleGAN
class DATASET_fits():
    def __init__(self,dataPath='',fineSize=512):
        super(DATASET_fits, self).__init__()
        # list all images into a list
        self.list = os.listdir(dataPath)
        self.list.sort()
        self.dataPath = dataPath
        self.fineSize = fineSize
    def __getitem__(self, index):
        path = os.path.join(self.dataPath,self.list[index])
        #read fits image
        imgsml = LoadSaveFits.read_fits(path)

        #read h5 image
        # h2 = h5py.File(path, 'r')
        # imgsml = h2['lenna'][:]

        hsize = 3721   #The number of PSF
        z1,h1,w1 = imgsml.shape
        img = np.zeros((hsize*2,h1,w1))
        img[hsize:,:,:] =imgsml[:,:,:]
        random_num = 60  #Determine the number of known PSFs
        for num in range(random_num):
            a = np.random.randint(0, hsize)
            img[a] = imgsml[a]
        z,h,w = img.shape
        img = img[:,int((h/2-self.fineSize/2)):int((h/2+self.fineSize/2)),int((w/2-self.fineSize/2)):int((w/2+self.fineSize/2))]
        img = torch.from_numpy(img)
        return img
    
    def __len__(self):
        # You should change 0 to the total size of your dataset.
        return len(self.list)

    




    
